@@ -1,6 +1,6 @@
-import asyncio
 import builtins
 import cProfile
+import dataclasses
 import functools
 import hashlib
 import heapq
@@ -9,7 +9,7 @@ import itertools
 import json
 import logging
 from concurrent import futures
-from typing import Callable, Iterable, Text, Type
+from typing import Any, Callable, Iterable, Text, Type, TypeVar
 
 import heapq_max
 import toolz
@@ -20,7 +20,7 @@ do_breakpoint = curried.do(lambda x: builtins.breakpoint())
 
 
 def do_if(condition, fun):
-    return curried.do(curried_ternary(condition, fun, toolz.identity))
+    return curried.do(ternary(condition, fun, toolz.identity))
 
 
 def check(condition, exception):
@@ -107,7 +107,7 @@ def ignore_input(inner):
     return ignore_and_run
 
 
-def curried_ternary(condition, f_true, f_false):
+def ternary(condition, f_true, f_false):
     def inner(*args, **kwargs):
         return (
             f_true(*args, **kwargs)
@@ -118,6 +118,10 @@ def curried_ternary(condition, f_true, f_false):
     return inner
 
 
+# Deprecated.
+curried_ternary = ternary
+
+
 def make_raise(exception):
     def inner():
         raise exception
@@ -152,99 +156,6 @@ def assert_that(f):
     return curried.do(_assert_f_output_on_inp(f))
 
 
-async def apipe(val, *funcs):
-    for f in funcs:
-        val = f(val)
-        if inspect.isawaitable(val):
-            val = await val
-    return val
-
-
-def acompose(*funcs):
-    async def composed(*args, **kwargs):
-        for f in reversed(funcs):
-            inp = f(*args, **kwargs)
-            if inspect.isawaitable(inp):
-                inp = await inp
-            args = [inp]
-            kwargs = {}
-        return inp
-
-    return composed
-
-
-def acompose_left(*funcs):
-    return acompose(*reversed(funcs))
-
-
-def run_sync(f):
-    """Runs a coroutine in a synchronous context, blocking until result arrives."""
-    loop = asyncio.new_event_loop()
-    return loop.run_until_complete(asyncio.ensure_future(f, loop=loop))
-
-
-@toolz.curry
-async def amap(f, it):
-    return await asyncio.gather(*map(f, it))
-
-
-@toolz.curry
-async def aexcepts(exception_type, func, handler, x):
-    try:
-        return await func(x)
-    except exception_type as error:
-        return handler(error)
-
-
-@toolz.curry
-async def mapa(f, it):
-    async for element in it:
-        yield f(element)
-
-
-async def aconcat(async_generators):
-    async for g in async_generators:
-        for x in g:
-            yield x
-
-
-def ajuxt(*funcs):
-    async def ajuxt_inner(x):
-        results = []
-        for f in funcs:
-            result = f(x)
-            if inspect.isawaitable(result):
-                result = await result
-            results.append(result)
-        return tuple(results)
-
-    return ajuxt_inner
-
-
-@toolz.curry
-async def afilter(func, it):
-    it = tuple(it)
-    results = await amap(func, it)
-    return toolz.pipe(
-        zip(it, results), curried.filter(toolz.second), curried.map(toolz.first)
-    )
-
-
-def afirst(*funcs, exception_type):
-    async def afirst_inner(x):
-        for f in funcs:
-            try:
-                result = f(x)
-                if inspect.isawaitable(result):
-                    result = await result
-                return result
-            except exception_type:
-                pass
-        raise exception_type
-
-    return afirst_inner
-
-
 @toolz.curry
 def pmap(f, n_workers, it):
     # The `tuple` is for callers convenience (even without it, the pool is eager).
@@ -346,17 +257,10 @@ def pair_right(f, element):
     return element, f(element)
 
 
-@toolz.curry
-async def apair_with(f, element):
-    return await f(element), element
-
-
-@toolz.curry
-async def apair_right(f, element):
-    return element, await f(element)
-
-
-average = toolz.compose_left(bifurcate(sum, toolz.count), star(operator.truediv))
+average = toolz.compose_left(
+    bifurcate(sum, toolz.count),
+    toolz.excepts(ZeroDivisionError, star(operator.truediv), lambda _: 0),
+)
 
 
 @toolz.curry
@@ -376,6 +280,10 @@ def wrap_tuple(x):
     return (x,)
 
 
+def invoke(x):
+    return x()
+
+
 @toolz.curry
 def assoc_in(d, keys, value, factory=dict):
     return update_in(d, keys, lambda x: value, value, factory)
@@ -411,3 +319,33 @@ def update_in(d, keys, func, default=None, factory=dict):
     else:
         inner[k] = func(default)
     return rv
+
+
+@toolz.curry
+def dataclass_transform(
+    attr_name: Text, attr_transformer: Callable[[Any], Any], dataclass_instance
+):
+    return dataclasses.replace(
+        dataclass_instance,
+        **{
+            attr_name: toolz.pipe(
+                dataclass_instance, operator.attrgetter(attr_name), attr_transformer
+            )
+        },
+    )
+
+
+@toolz.curry
+def dataclass_replace(attr_name: Text, attr_value: Any, dataclass_instance):
+    return dataclasses.replace(dataclass_instance, **{attr_name: attr_value})
+
+
+_R = TypeVar("_R")
+_E = TypeVar("_E")
+
+
+@toolz.curry
+def reduce(
+    reducer: Callable[[_R, _E], _R], initial_value: _R, elements: Iterable[_E]
+) -> _R:
+    return functools.reduce(reducer, elements, initial_value)
