@@ -105,19 +105,12 @@ def groupby_many(f, it):
 
 
 @toolz.curry
-async def agroupby_many(f, it):
-    return await functional.apipe(
-        it,
-        functional.amap(
-            functional.acompose_left(
-                functional.apair_with(f),
-                functional.star(lambda x, y: (x, [y])),
-                functional.star(itertools.product),
-            )
-        ),
-        toolz.concat,
-        edges_to_graph,
-    )
+def groupby_many_reduce(key, reducer, seq):
+    result = {}
+    for element in seq:
+        for key_result in key(element):
+            result[key_result] = reducer(result.get(key_result, None), element)
+    return result
 
 
 @toolz.curry
